<?php
  // DHCP is defined in RFC 2131

  define('DHCP_CLIENT_PORT', 68);
  define('DHCP_SERVER_PORT', 67);
  define('DHCP_BROADCAST', '255.255.255.255');
  define('DHCP_ZERO', '0.0.0.0');

  define('DHCP_DISCOVER', 1);
  define('DHCP_OFFER', 2);
  define('DHCP_REQUEST', 3);
  define('DHCP_DECLINE', 4);
  define('DHCP_ACK', 5);
  define('DHCP_NACK', 6);
  define('DHCP_RELEASE', 7);
  define('DHCP_INFORM', 8);
  define('DHCP_FORCE_RENEW', 9);

  define('DHCP_BROADCAST_FLAG', 32768);

  // RFC 951
  define('DHCP_BOOTREQUEST', 1);
  define('DHCP_BOOTREPLY', 2);

  // http://www.iana.org/assignments/arp-parameters
  define('DHCP_HARDWARE_TYPE_ETHERNET_10',	1);
  define('DHCP_HARDWARE_TYPE_ETHERNET_3',	2);
  define('DHCP_HARDWARE_TYPE_AMATEUR_RADIO',	3);
  define('DHCP_HARDWARE_TYPE_TOKEN_RING',	4);
  define('DHCP_HARDWARE_TYPE_CHAOS',		5);
  define('DHCP_HARDWARE_TYPE_IEEE_802',		6);
  define('DHCP_HARDWARE_TYPE_ARCNET',		7);
  define('DHCP_HARDWARE_TYPE_HYPERCHANNEL',	8);
  define('DHCP_HARDWARE_TYPE_LANSTAR', 		9);
  define('DHCP_HARDWARE_TYPE_AUTONET', 		10);
  define('DHCP_HARDWARE_TYPE_LOCALTALK', 	11);
  define('DHCP_HARDWARE_TYPE_LOCALNET', 	12);
  define('DHCP_HARDWARE_TYPE_ULTRA_LINK', 	13);
  define('DHCP_HARDWARE_TYPE_SMDS', 		14);
  define('DHCP_HARDWARE_TYPE_FRAME_RELAY', 	15);
  define('DHCP_HARDWARE_TYPE_ATM_1', 		16);
  define('DHCP_HARDWARE_TYPE_HDLC', 		17);
  define('DHCP_HARDWARE_TYPE_FIBRE_CHANNEL', 	18);
  define('DHCP_HARDWARE_TYPE_ATM_2', 		19);
  define('DHCP_HARDWARE_TYPE_SERIAL', 		20);
  define('DHCP_HARDWARE_TYPE_ATM_3', 		21);
  define('DHCP_HARDWARE_TYPE_MIL_STD', 		22);
  define('DHCP_HARDWARE_TYPE_METRICOM', 	23);
  define('DHCP_HARDWARE_TYPE_IEEE_1394', 	24);
  define('DHCP_HARDWARE_TYPE_MAPOS', 		25);
  define('DHCP_HARDWARE_TYPE_TWINAXIAL', 	26);
  define('DHCP_HARDWARE_TYPE_EUI_64', 		27);
  define('DHCP_HARDWARE_TYPE_HIPARP', 		28);
  define('DHCP_HARDWARE_TYPE_IP_ARP_ISO7816_3',	29);
  define('DHCP_HARDWARE_TYPE_ARPSEC', 		30);
  define('DHCP_HARDWARE_TYPE_IPSEC_TUNNEL', 	31);
  define('DHCP_HARDWARE_TYPE_INFINIBAND',	32);
  define('DHCP_HARDWARE_TYPE_CAI', 		33);

  class dhcp
  {
    var $op = DHCP_BOOTREQUEST;				// Op code
    var $htype = DHCP_HARDWARE_TYPE_ETHERNET_10;	// HW Address Type
    var $hlen = 6;					// Hardware Address Length
    var $hops = 0;					// Hw Options
    var $xid;						// Transaction ID
    var $secs = 0;				 	// up time
    var $flags = 0;					// flags
    var $ciaddr = DHCP_ZERO;				// Client IP
    var $yiaddr = DHCP_ZERO;				// Your Client IP
    var $siaddr = DHCP_ZERO;				// Server IP
    var $giaddr = DHCP_ZERO;				// Relay Agent IP
    var $chaddr = '';					// Client HW Address
    var $sname = '';					// Optional server host name
    var $file = '';					// Boot file name
    var $options;					// DHCP Options
    var $dest = DHCP_BROADCAST;				// Server IP Address
    var $port = DHCP_SERVER_PORT;			// Server Port

    function dhcp($dest=DHCP_BROADCAST, $port=DHCP_SERVER_PORT)
    {
      $this->dest = $dest;
      $this->port = $port;
      $this->options = new dhcp_options();
      $this->xid = rand(); 
    }

    function set_hw_address($mac)
    {
      $mac = strtoupper($mac);

      $mac = str_replace('-', ':', $mac);
      $mac = str_replace('\.', ':', $mac);
      $mac = str_replace(' ', ':', trim($mac));

      // strip out non hex chars
      $mac = preg_replace('%[^0-9A-F:]%', '', $mac);

      if(strpos($mac, ':'))
      {
        $m = explode(':', $mac);
        $mac = '';
        for($i = 0; $i < count($m); $i++)
        {
          while(strlen($m[$i]) < 2)
            $m[$i] = '0' . $m[$i];
          $mac .= $m[$i];
        }
      }

      $this->chaddr = '';
      for($i = 0; $i < strlen($mac); $i+=2)
        $this->chaddr .= chr(hexdec(substr($mac, $i, 2)));
      $this->hlen = strlen($this->chaddr);
    }

    function get_hw_address()
    {
      $z = array();
      for($i = 0; $i < $this->hlen; $i++)
        $z[] = str_pad(dechex(ord($this->chaddr{$i})), 2, '0', STR_PAD_LEFT);
      return join(':', $z);
    }

    function decode($data)
    {
      $z = unpack('C1op/C1htype/C1hlen/C1hops/N1xid/n1secs/n1flags/N1ciaddr/N1yiaddr/N1siaddr/N1giaddr', $data);
      $this->op = $z['op'];
      $this->htype = $z['htype'];
      $this->hlen = $z['hlen'];
      $this->hops = $z['hops'];
      $this->xid = $z['xid'];
      $this->secs = $z['secs'];
      $this->flags = $z['flags'];
      $this->ciaddr = long2ip($z['ciaddr']);
      $this->yiaddr = long2ip($z['yiaddr']);
      $this->siaddr = long2ip($z['siaddr']);
      $this->giaddr = long2ip($z['giaddr']);
      $this->chaddr = substr($data, 28, 16);
      $this->sname = substr($data, 44, 64);
      $this->file = substr($data, 108, 128);
      $this->options->decode(substr($data, 236));
    }

    function encode()
    {
      $ret = chr($this->op) . chr($this->htype) . chr($this->hlen) . chr($this->hops);
      $ret .= pack('Nnn', $this->xid, $this->secs, $this->flags);
      $ret .= pack('NN', ip2long($this->ciaddr), ip2long($this->yiaddr));
      $ret .= pack('NN', ip2long($this->siaddr), ip2long($this->giaddr));
      $ret .= str_pad($this->chaddr, 16, chr(0));
      $ret .= str_pad($this->sname, 64, chr(0));
      $ret .= str_pad($this->file, 128, chr(0));
      $ret .= $this->options->encode();
      return $ret;
    }

    function send()
    {
      $packet = $this->encode();
      $sock = socket_create(AF_INET, SOCK_DGRAM, SOL_UDP);
      if(!$sock)
        trigger_error('Unable to create socket: ' . socket_last_error(), E_USER_ERROR);
      if($this->dest == DHCP_BROADCAST)
      {
        if(!socket_set_option($sock, SOL_SOCKET, SO_BROADCAST, TRUE))
          trigger_error('Unable to set SO_BROADCAST flag: ' . socket_last_error(), E_USER_ERROR);
      }
      if(!socket_sendto($sock, $packet, strlen($packet), 0, $this->dest, $this->port))
        trigger_error('Unable to send packet: ' . socket_last_error(), E_USER_ERROR);
      socket_close($sock);
    }

    function getmicrotime()
    {
      list($usec, $sec) = explode(' ',microtime());
      return ((float)$usec + (float)$sec);
    }

    function receive()
    {
      $sock = socket_create(AF_INET, SOCK_DGRAM, SOL_UDP);
      if(!$sock)
        trigger_error('Unable to create socket: ' . socket_last_error(), E_USER_ERROR);
      if(!socket_bind($sock, $this->dest, $this->port))
        trigger_error('Unable to bind to socket: ' . socket_last_error(), E_USER_ERROR);

      $packet = NULL;
      $port = $this->port;
      $host = NULL;
      if(socket_recvfrom($sock, $packet, 4096, 0, $host, $port))
      {
        echo "Recieved from $host:$port\n";
        $this->decode($packet);
        $this->display();
      }
      socket_close($sock);
    }

    function display()
    {
      if($this->op == DHCP_BOOTREQUEST)
        echo "Boot Request:\n";
      elseif($this->op == DHCP_BOOTREPLY)
        echo "Boot Reply:\n";
      else
        echo "Unknown Op={$this->op}:\n";

      echo '  Hardware Type: ';
      switch($this->htype)
      {
        case 1: echo "Ethernet (10Mb)\n"; break;
        case 2: echo "Experimental Ethernet (3Mb)\n"; break;
        case 3: echo "Amateur Radio AX.25\n"; break;
        case 4: echo "Proteon ProNET Token Ring\n"; break;
        case 5: echo "Chaos\n"; break;
        case 6: echo "IEEE 802 Networks\n"; break;
        case 7: echo "ARCNET\n"; break;
        case 8: echo "Hyperchannel\n"; break;
        case 9: echo "Lanstar\n"; break;
        case 10: echo "Autonet Short Address\n"; break;
        case 11: echo "LocalTalk\n"; break;
        case 12: echo "LocalNet (IBM PCNet or SYTEK LocalNET)\n"; break;
        case 13: echo "Ultra link\n"; break;
        case 14: echo "SMDS\n"; break;
        case 15: echo "Frame Relay\n"; break;
        case 16: echo "Asynchronous Transmission Mode (ATM)\n"; break;
        case 17: echo "HDLC\n"; break;
        case 18: echo "Fibre Channel\n"; break;
        case 19: echo "Asynchronous Transmission Mode (ATM)\n"; break;
        case 20: echo "Serial Line\n"; break;
        case 21: echo "Asynchronous Transmission Mode (ATM)\n"; break;
        case 22: echo "MIL-STD-188-220\n"; break;
        case 23: echo "Metricom\n"; break;
        case 24: echo "IEEE 1394.1995\n"; break;
        case 25: echo "MAPOS\n"; break;
        case 26: echo "Twinaxial\n"; break;
        case 27: echo "EUI-64\n"; break;
        case 28: echo "HIPARP\n"; break;
        case 29: echo "IP and ARP over ISO 7816-3\n"; break;
        case 30: echo "ARPSec\n"; break;
        case 31: echo "IPsec tunnel\n"; break;
        case 32: echo "InfiniBand (TM)\n"; break;
        case 33: echo "TIA-102 Project 25 Common Air Interface (CAI)\n"; break;
        default: echo "Unknown\n";
      }

      echo "  Hardware Address Length: {$this->hlen}\n";
      echo "  Hardware Options: {$this->hops}\n";
      echo "  Transaction ID: {$this->xid}\n";
      echo "  Uptime: {$this->secs}\n";
      echo "  Flags: {$this->flags}\n";
      echo "  Client IP: {$this->ciaddr}\n";
      echo "  Your IP: {$this->yiaddr}\n";
      echo "  Server IP: {$this->siaddr}\n";
      echo "  Relay IP: {$this->giaddr}\n";
      echo "  Hardware Address: " . $this->get_hw_address() . "\n";
      echo "  Bootfile: {$this->file}\n";
      echo "  Options:\n";
      $this->options->display();
      echo "  Port: {$this->port}\n";
      echo "  Dest IP: {$this->dest}\n\n";
    }
  }

  // http://www.iana.org/assignments/bootp-dhcp-parameters
  define('DHCP_OPTION_PAD', 0);
  define('DHCP_OPTION_SUBNET_MASK', 1);
  define('DHCP_OPTION_TIME_OFFSET', 2);
  define('DHCP_OPTION_ROUTER', 3);
  define('DHCP_OPTION_TIME_SERVER', 4);
  define('DHCP_OPTION_NAME_SERVER', 5);
  define('DHCP_OPTION_DOMAIN_SERVER', 6);
  define('DHCP_OPTION_LOG_SERVER', 7);
  define('DHCP_OPTION_QUOTES_SERVER', 8);
  define('DHCP_OPTION_LPR_SERVER', 9);
  define('DHCP_OPTION_IMPRESS_SERVER', 10);
  define('DHCP_OPTION_RLP_SERVER', 11);
  define('DHCP_OPTION_HOSTNAME', 12);
  define('DHCP_OPTION_BOOT_FILE_SIZE', 13);
  define('DHCP_OPTION_MERIT_DUMP_FILE', 14);
  define('DHCP_OPTION_DOMAIN_NAME', 15);
  define('DHCP_OPTION_SWAP_SERVER', 16);
  define('DHCP_OPTION_ROOT_PATH', 17);
  define('DHCP_OPTION_EXTENSION_FILE', 18);
  define('DHCP_OPTION_FORWARD_ON/OFF', 19);
  define('DHCP_OPTION_SRCRTE_ON/OFF', 20);
  define('DHCP_OPTION_POLICY_FILTER', 21);
  define('DHCP_OPTION_MAX_DG_ASSEMBLY', 22);
  define('DHCP_OPTION_DEFAULT_IP_TTL', 23);
  define('DHCP_OPTION_MTU_TIMEOUT', 24);
  define('DHCP_OPTION_MTU_PLATEAU', 25);
  define('DHCP_OPTION_MTU_INTERFACE', 26);
  define('DHCP_OPTION_MTU_SUBNET', 27);
  define('DHCP_OPTION_BROADCAST_ADDRESS', 28);
  define('DHCP_OPTION_MASK_DISCOVERY', 29);
  define('DHCP_OPTION_MASK_SUPPLIER', 30);
  define('DHCP_OPTION_ROUTER_DISCOVERY', 31);
  define('DHCP_OPTION_ROUTER_REQUEST', 32);
  define('DHCP_OPTION_STATIC_ROUTE', 33);
  define('DHCP_OPTION_TRAILERS', 34);
  define('DHCP_OPTION_ARP_TIMEOUT', 35);
  define('DHCP_OPTION_ETHERNET', 36);
  define('DHCP_OPTION_DEFAULT_TCP_TTL', 37);
  define('DHCP_OPTION_KEEPALIVE_TIME', 38);
  define('DHCP_OPTION_KEEPALIVE_DATA', 39);
  define('DHCP_OPTION_NIS_DOMAIN', 40);
  define('DHCP_OPTION_NIS_SERVERS', 41);
  define('DHCP_OPTION_NTP_SERVERS', 42);
  define('DHCP_OPTION_VENDOR_SPECIFIC', 43);
  define('DHCP_OPTION_NETBIOS_NAME_SRV', 44);
  define('DHCP_OPTION_NETBIOS_DIST_SRV', 45);
  define('DHCP_OPTION_NETBIOS_NODE_TYPE', 46);
  define('DHCP_OPTION_NETBIOS_SCOPE', 47);
  define('DHCP_OPTION_X_WINDOW_FONT', 48);
  define('DHCP_OPTION_X_WINDOW_MANAGER', 49);
  define('DHCP_OPTION_ADDRESS_REQUEST', 50);
  define('DHCP_OPTION_ADDRESS_TIME', 51);
  define('DHCP_OPTION_OVERLOAD', 52);
  define('DHCP_OPTION_MESSAGE_TYPE', 53);
  define('DHCP_OPTION_DHCP_SERVER_ID', 54);
  define('DHCP_OPTION_PARAMETER_LIST', 55);
  define('DHCP_OPTION_DHCP_MESSAGE', 56);
  define('DHCP_OPTION_DHCP_MAX_MSG_SIZE', 57);
  define('DHCP_OPTION_RENEWAL_TIME', 58);
  define('DHCP_OPTION_REBINDING_TIME', 59);
  define('DHCP_OPTION_CLASS_ID', 60);
  define('DHCP_OPTION_CLIENT_ID', 61);
  define('DHCP_OPTION_NETWARE/IP_DOMAIN', 62);
  define('DHCP_OPTION_NETWARE/IP_OPTION', 63);
  define('DHCP_OPTION_NIS_DOMAIN_NAME', 64);
  define('DHCP_OPTION_NIS_SERVER_ADDR', 65);
  define('DHCP_OPTION_SERVER_NAME', 66);
  define('DHCP_OPTION_BOOTFILE_NAME', 67);
  define('DHCP_OPTION_HOME_AGENT_ADDRS', 68);
  define('DHCP_OPTION_SMTP_SERVER', 69);
  define('DHCP_OPTION_POP3_SERVER', 70);
  define('DHCP_OPTION_NNTP_SERVER', 71);
  define('DHCP_OPTION_WWW_SERVER', 72);
  define('DHCP_OPTION_FINGER_SERVER', 73);
  define('DHCP_OPTION_IRC_SERVER', 74);
  define('DHCP_OPTION_STREETTALK_SERVER', 75);
  define('DHCP_OPTION_STDA_SERVER', 76);
  define('DHCP_OPTION_USER_CLASS', 77);
  define('DHCP_OPTION_DIRECTORY_AGENT', 78);
  define('DHCP_OPTION_SERVICE_SCOPE', 79);
  define('DHCP_OPTION_RAPID_COMMIT', 80);
  define('DHCP_OPTION_CLIENT_FQDN', 81);
  define('DHCP_OPTION_RELAY_AGENT_INFORMATION', 82);
  define('DHCP_OPTION_ISNS', 83);
  define('DHCP_OPTION_NDS_SERVERS', 85);
  define('DHCP_OPTION_NDS_TREE_NAME', 86);
  define('DHCP_OPTION_NDS_CONTEXT', 87);
  define('DHCP_OPTION_AUTHENTICATION', 90);
  define('DHCP_OPTION_CLIENT_SYSTEM', 93);
  define('DHCP_OPTION_CLIENT_NDI', 94);
  define('DHCP_OPTION_LDAP', 95);
  define('DHCP_OPTION_UUID/GUID', 97);
  define('DHCP_OPTION_USER_AUTH', 98);
  define('DHCP_OPTION_NETINFO_ADDRESS', 112);
  define('DHCP_OPTION_NETINFO_TAG', 113);
  define('DHCP_OPTION_URL', 114);
  define('DHCP_OPTION_AUTO_CONFIG', 116);
  define('DHCP_OPTION_NAME_SERVICE_SEARCH', 117);
  define('DHCP_OPTION_SUBNET_SELECTION_OPTION', 118);
  define('DHCP_OPTION_DOMAIN_SEARCH', 119);
  define('DHCP_OPTION_SIP_SERVERS_DHCP_OPTION', 120);
  define('DHCP_OPTION_CLASSLESS_STATIC_ROUTE', 121);
  define('DHCP_OPTION_CCC', 122);
  define('DHCP_OPTION_GEOCONF_OPTION', 123);
  define('DHCP_OPTION_END', 255);

  define('DHCP_OPTIONS_STRING', 0);
  define('DHCP_OPTIONS_BYTE', 1);
  define('DHCP_OPTIONS_SHORT', 2);
  define('DHCP_OPTIONS_BINARY', 3);
  define('DHCP_OPTIONS_LONG', 4);
  define('DHCP_OPTIONS_IP', 5);

  class dhcp_options
  {
    var $options = array();
    var $size = array();
    var $types = array();
    var $names = array();

    function dhcp_options()
    {
      $this->size = array(0=>0, 1=>4, 2=>4, 13=>2, 19=>1, 20=>1, 22=>2, 23=>1, 24=>4, 26=>2, 27=>1, 28=>4, 29=>1, 30=>1,
                          31=>1, 32=>4, 34=>1, 35=>4, 36=>1, 37=>1, 38=>4, 39=>1, 46=>1, 50=>4, 51=>4, 52=>1, 53=>1, 54=>4,
                          57=>2, 58=>4, 59=>4, 80=>0, 118=>4, 123=>16, 255=>0);
      $this->types = array(1=>DHCP_OPTIONS_IP, 2=>DHCP_OPTIONS_LONG, 3=>DHCP_OPTIONS_IP, 4=>DHCP_OPTIONS_IP,
                           5=>DHCP_OPTIONS_IP, 6=>DHCP_OPTIONS_IP, 7=>DHCP_OPTIONS_IP, 8=>DHCP_OPTIONS_IP,
                           9=>DHCP_OPTIONS_IP, 10=>DHCP_OPTIONS_IP, 11=>DHCP_OPTIONS_IP, 13=>DHCP_OPTIONS_SHORT,
                           19=>DHCP_OPTIONS_BYTE, 20=>DHCP_OPTIONS_BYTE, 22=>DHCP_OPTIONS_SHORT, 23=>DHCP_OPTIONS_BYTE,
                           24=>DHCP_OPTIONS_LONG, 26=>DHCP_OPTIONS_SHORT, 27=>DHCP_OPTIONS_BYTE, 28=>DHCP_OPTIONS_IP,
                           29=>DHCP_OPTIONS_BYTE, 30=>DHCP_OPTIONS_BYTE, 31=>DHCP_OPTIONS_BYTE, 32=>DHCP_OPTIONS_IP,
                           34=>DHCP_OPTIONS_BYTE, 35=>DHCP_OPTIONS_LONG, 36=>DHCP_OPTIONS_BYTE, 37=>DHCP_OPTIONS_BYTE,
                           38=>DHCP_OPTIONS_LONG, 39=>DHCP_OPTIONS_BYTE, 46=>DHCP_OPTIONS_BYTE, 50=>DHCP_OPTIONS_IP,
                           51=>DHCP_OPTIONS_LONG, 52=>DHCP_OPTIONS_BYTE, 53=>DHCP_OPTIONS_BYTE, 54=>DHCP_OPTIONS_LONG,
                           55=>DHCP_OPTIONS_BINARY, 57=>DHCP_OPTIONS_SHORT, 58=>DHCP_OPTIONS_LONG, 59=>DHCP_OPTIONS_LONG,
                           61=>DHCP_OPTIONS_BINARY, 116=>DHCP_OPTIONS_BINARY, 118=>DHCP_OPTIONS_IP);

      // generate constant names
      foreach(get_defined_constants() as $key=>$value)
        if(substr($key, 0, 12) == 'DHCP_OPTION_')
          $this->names[$value] = substr($key, 12);
    }

    function get_type($code, $type=NULL)
    {
      if(is_null($type))
      {
        if(isset($this->types[$code]))
          $type = $this->types[$code];
        else
          $type = DHCP_OPTIONS_STRING;
      }
      elseif(!isset($this->types[$code]))
        $this->types[$code] = $type;
      return $type;
    }

    function set($code, $value, $type=NULL)
    {
      $type = $this->get_type($code, $type);

      // encode value according to type
      if($type == DHCP_OPTIONS_BYTE)  $val = chr($value);
      elseif($type == DHCP_OPTIONS_SHORT) $val = pack('n', $value);
      elseif($type == DHCP_OPTIONS_LONG)  $val = pack('N', $value);
      elseif($type == DHCP_OPTIONS_IP)
      {
        if(!is_array($value)) $value = explode(' ', $value);
        $val = '';
        foreach($value as $v)
        {
          $v = explode('.', $v);
          $val .= chr($v{0}) . chr($v{1}) . chr($v{2}) . chr($v{3});
        }
      }
      elseif($type == DHCP_OPTIONS_BINARY)
      {
        $val = '';
        foreach(explode(':', $value) as $byte)
          $val .= chr(hexdec($byte));
      }
      else
        $val = $value;

      // check size limits
      if(isset($this->size[$code]) && strlen($val) > $this->size[$code])
      {
        if(!isset($this->names[$code])) $this->names[$code] = "Unknown($code)";
        if(is_array($value)) $value = join(', ', $value);
        trigger_error("Value too long for {$this->names[$code]}: $value", E_USER_WARNING);
      }

      // store in array
      $this->options[$code] = $val;
    }

    function get($code, $type=NULL)
    {
      $type = $this->get_type($code, $type);

      if($type == DHCP_OPTIONS_BYTE) $value = ord($this->options[$code]);
      elseif($type == DHCP_OPTIONS_SHORT)
      {
        $value = unpack('n1z', $this->options[$code]);
        $value = $value['z'];
      }
      elseif($type == DHCP_OPTIONS_LONG)
      {
        $value = unpack('N1z', $this->options[$code]);
        $value = $value['z'];
      }
      elseif($type == DHCP_OPTIONS_IP)
      {
        $value = '';
        for($i = 0; $i < strlen($this->options[$code]); $i++)
          $value .= ord($this->options[$code]{$i}) . (($i % 4 == 3) ? ' ' : '.');
      }
      elseif($type == DHCP_OPTIONS_BINARY)
      {
        $value = '';
        for($i = 0; $i < strlen($this->options[$code]); $i++)
          $value .= ':' . str_pad(dechex(ord($this->options[$code]{$i})), 2, '0', STR_PAD_LEFT);
        $value = substr($value, 1);
      }
      else
        $value = $this->options[$code];

      // Special case
      if($code == DHCP_OPTION_MESSAGE_TYPE)
      {
        switch($value)
        {
          case DHCP_DISCOVER:  $value = 'DISCOVER'; break;
          case DHCP_OFFER:     $value = 'OFFER'; break;
          case DHCP_REQUEST:   $value = 'REQUEST'; break;
          case DHCP_DECLINE:   $value = 'DECLINE'; break;
          case DHCP_ACK:       $value = 'ACK'; break;
          case DHCP_NACK:      $value = 'NACK'; break;
          case DHCP_RELEASE:   $value = 'RELEASE'; break;
          case DHCP_INFORM:    $value = 'INFORM'; break;
          case DHCP_FORCE_RENEW: $value = 'FORCE_RENEW'; break;
        }
      }

      return $value;
    }

    function encode()
    {
      $ret = chr(99) . chr(130) . chr(83) . chr(99); // magic cookie
      foreach($this->options as $code=>$value)
      {
        $ret .= chr($code);
        if(isset($this->size[$code]))
          $value .= str_pad(substr($value, 0, $this->size[$code]), $this->size[$code], chr(0));
        $ret .= chr(strlen($value)) . $value;
      }
      $ret .= chr(DHCP_OPTION_END);
      return $ret;
    }

    function decode($data)
    {
      $this->options = array();
      $pos = 4; // skip magic cookie
      while($pos < strlen($data) && $data{$pos} != chr(DHCP_OPTION_END))
      {
        $code = ord($data{$pos++});
        $size = ord($data{$pos++});
        $this->options[$code] = substr($data, $pos, $size);
        $pos += $size;
      }
    }

    function display()
    {
      foreach($this->options as $code=>$value)
      {
        $value = $this->get($code);
        echo "\t{$this->names[$code]} = $value\n";
      }
    }
  }
?>
